"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AminoConverter = void 0;
var _helpers = require("../../../helpers");
//@ts-nocheck

var AminoConverter = {
  "/kava.cdp.v1beta1.MsgCreateCDP": {
    aminoType: "/kava.cdp.v1beta1.MsgCreateCDP",
    toAmino: function toAmino(_ref) {
      var sender = _ref.sender,
        collateral = _ref.collateral,
        principal = _ref.principal,
        collateralType = _ref.collateralType;
      return {
        sender: sender,
        collateral: {
          denom: collateral.denom,
          amount: _helpers.Long.fromValue(collateral.amount).toString()
        },
        principal: {
          denom: principal.denom,
          amount: _helpers.Long.fromValue(principal.amount).toString()
        },
        collateral_type: collateralType
      };
    },
    fromAmino: function fromAmino(_ref2) {
      var sender = _ref2.sender,
        collateral = _ref2.collateral,
        principal = _ref2.principal,
        collateral_type = _ref2.collateral_type;
      return {
        sender: sender,
        collateral: {
          denom: collateral.denom,
          amount: collateral.amount
        },
        principal: {
          denom: principal.denom,
          amount: principal.amount
        },
        collateralType: collateral_type
      };
    }
  },
  "/kava.cdp.v1beta1.MsgDeposit": {
    aminoType: "/kava.cdp.v1beta1.MsgDeposit",
    toAmino: function toAmino(_ref3) {
      var depositor = _ref3.depositor,
        owner = _ref3.owner,
        collateral = _ref3.collateral,
        collateralType = _ref3.collateralType;
      return {
        depositor: depositor,
        owner: owner,
        collateral: {
          denom: collateral.denom,
          amount: _helpers.Long.fromValue(collateral.amount).toString()
        },
        collateral_type: collateralType
      };
    },
    fromAmino: function fromAmino(_ref4) {
      var depositor = _ref4.depositor,
        owner = _ref4.owner,
        collateral = _ref4.collateral,
        collateral_type = _ref4.collateral_type;
      return {
        depositor: depositor,
        owner: owner,
        collateral: {
          denom: collateral.denom,
          amount: collateral.amount
        },
        collateralType: collateral_type
      };
    }
  },
  "/kava.cdp.v1beta1.MsgWithdraw": {
    aminoType: "/kava.cdp.v1beta1.MsgWithdraw",
    toAmino: function toAmino(_ref5) {
      var depositor = _ref5.depositor,
        owner = _ref5.owner,
        collateral = _ref5.collateral,
        collateralType = _ref5.collateralType;
      return {
        depositor: depositor,
        owner: owner,
        collateral: {
          denom: collateral.denom,
          amount: _helpers.Long.fromValue(collateral.amount).toString()
        },
        collateral_type: collateralType
      };
    },
    fromAmino: function fromAmino(_ref6) {
      var depositor = _ref6.depositor,
        owner = _ref6.owner,
        collateral = _ref6.collateral,
        collateral_type = _ref6.collateral_type;
      return {
        depositor: depositor,
        owner: owner,
        collateral: {
          denom: collateral.denom,
          amount: collateral.amount
        },
        collateralType: collateral_type
      };
    }
  },
  "/kava.cdp.v1beta1.MsgDrawDebt": {
    aminoType: "/kava.cdp.v1beta1.MsgDrawDebt",
    toAmino: function toAmino(_ref7) {
      var sender = _ref7.sender,
        collateralType = _ref7.collateralType,
        principal = _ref7.principal;
      return {
        sender: sender,
        collateral_type: collateralType,
        principal: {
          denom: principal.denom,
          amount: _helpers.Long.fromValue(principal.amount).toString()
        }
      };
    },
    fromAmino: function fromAmino(_ref8) {
      var sender = _ref8.sender,
        collateral_type = _ref8.collateral_type,
        principal = _ref8.principal;
      return {
        sender: sender,
        collateralType: collateral_type,
        principal: {
          denom: principal.denom,
          amount: principal.amount
        }
      };
    }
  },
  "/kava.cdp.v1beta1.MsgRepayDebt": {
    aminoType: "/kava.cdp.v1beta1.MsgRepayDebt",
    toAmino: function toAmino(_ref9) {
      var sender = _ref9.sender,
        collateralType = _ref9.collateralType,
        payment = _ref9.payment;
      return {
        sender: sender,
        collateral_type: collateralType,
        payment: {
          denom: payment.denom,
          amount: _helpers.Long.fromValue(payment.amount).toString()
        }
      };
    },
    fromAmino: function fromAmino(_ref10) {
      var sender = _ref10.sender,
        collateral_type = _ref10.collateral_type,
        payment = _ref10.payment;
      return {
        sender: sender,
        collateralType: collateral_type,
        payment: {
          denom: payment.denom,
          amount: payment.amount
        }
      };
    }
  },
  "/kava.cdp.v1beta1.MsgLiquidate": {
    aminoType: "/kava.cdp.v1beta1.MsgLiquidate",
    toAmino: function toAmino(_ref11) {
      var keeper = _ref11.keeper,
        borrower = _ref11.borrower,
        collateralType = _ref11.collateralType;
      return {
        keeper: keeper,
        borrower: borrower,
        collateral_type: collateralType
      };
    },
    fromAmino: function fromAmino(_ref12) {
      var keeper = _ref12.keeper,
        borrower = _ref12.borrower,
        collateral_type = _ref12.collateral_type;
      return {
        keeper: keeper,
        borrower: borrower,
        collateralType: collateral_type
      };
    }
  }
};
exports.AminoConverter = AminoConverter;