"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AminoConverter = void 0;
var _helpers = require("../../../helpers");
//@ts-nocheck

var AminoConverter = {
  "/kava.swap.v1beta1.MsgDeposit": {
    aminoType: "/kava.swap.v1beta1.MsgDeposit",
    toAmino: function toAmino(_ref) {
      var depositor = _ref.depositor,
        tokenA = _ref.tokenA,
        tokenB = _ref.tokenB,
        slippage = _ref.slippage,
        deadline = _ref.deadline;
      return {
        depositor: depositor,
        token_a: {
          denom: tokenA.denom,
          amount: _helpers.Long.fromValue(tokenA.amount).toString()
        },
        token_b: {
          denom: tokenB.denom,
          amount: _helpers.Long.fromValue(tokenB.amount).toString()
        },
        slippage: slippage,
        deadline: deadline.toString()
      };
    },
    fromAmino: function fromAmino(_ref2) {
      var depositor = _ref2.depositor,
        token_a = _ref2.token_a,
        token_b = _ref2.token_b,
        slippage = _ref2.slippage,
        deadline = _ref2.deadline;
      return {
        depositor: depositor,
        tokenA: {
          denom: token_a.denom,
          amount: token_a.amount
        },
        tokenB: {
          denom: token_b.denom,
          amount: token_b.amount
        },
        slippage: slippage,
        deadline: _helpers.Long.fromString(deadline)
      };
    }
  },
  "/kava.swap.v1beta1.MsgWithdraw": {
    aminoType: "/kava.swap.v1beta1.MsgWithdraw",
    toAmino: function toAmino(_ref3) {
      var from = _ref3.from,
        shares = _ref3.shares,
        minTokenA = _ref3.minTokenA,
        minTokenB = _ref3.minTokenB,
        deadline = _ref3.deadline;
      return {
        from: from,
        shares: shares,
        min_token_a: {
          denom: minTokenA.denom,
          amount: _helpers.Long.fromValue(minTokenA.amount).toString()
        },
        min_token_b: {
          denom: minTokenB.denom,
          amount: _helpers.Long.fromValue(minTokenB.amount).toString()
        },
        deadline: deadline.toString()
      };
    },
    fromAmino: function fromAmino(_ref4) {
      var from = _ref4.from,
        shares = _ref4.shares,
        min_token_a = _ref4.min_token_a,
        min_token_b = _ref4.min_token_b,
        deadline = _ref4.deadline;
      return {
        from: from,
        shares: shares,
        minTokenA: {
          denom: min_token_a.denom,
          amount: min_token_a.amount
        },
        minTokenB: {
          denom: min_token_b.denom,
          amount: min_token_b.amount
        },
        deadline: _helpers.Long.fromString(deadline)
      };
    }
  },
  "/kava.swap.v1beta1.MsgSwapExactForTokens": {
    aminoType: "/kava.swap.v1beta1.MsgSwapExactForTokens",
    toAmino: function toAmino(_ref5) {
      var requester = _ref5.requester,
        exactTokenA = _ref5.exactTokenA,
        tokenB = _ref5.tokenB,
        slippage = _ref5.slippage,
        deadline = _ref5.deadline;
      return {
        requester: requester,
        exact_token_a: {
          denom: exactTokenA.denom,
          amount: _helpers.Long.fromValue(exactTokenA.amount).toString()
        },
        token_b: {
          denom: tokenB.denom,
          amount: _helpers.Long.fromValue(tokenB.amount).toString()
        },
        slippage: slippage,
        deadline: deadline.toString()
      };
    },
    fromAmino: function fromAmino(_ref6) {
      var requester = _ref6.requester,
        exact_token_a = _ref6.exact_token_a,
        token_b = _ref6.token_b,
        slippage = _ref6.slippage,
        deadline = _ref6.deadline;
      return {
        requester: requester,
        exactTokenA: {
          denom: exact_token_a.denom,
          amount: exact_token_a.amount
        },
        tokenB: {
          denom: token_b.denom,
          amount: token_b.amount
        },
        slippage: slippage,
        deadline: _helpers.Long.fromString(deadline)
      };
    }
  },
  "/kava.swap.v1beta1.MsgSwapForExactTokens": {
    aminoType: "/kava.swap.v1beta1.MsgSwapForExactTokens",
    toAmino: function toAmino(_ref7) {
      var requester = _ref7.requester,
        tokenA = _ref7.tokenA,
        exactTokenB = _ref7.exactTokenB,
        slippage = _ref7.slippage,
        deadline = _ref7.deadline;
      return {
        requester: requester,
        token_a: {
          denom: tokenA.denom,
          amount: _helpers.Long.fromValue(tokenA.amount).toString()
        },
        exact_token_b: {
          denom: exactTokenB.denom,
          amount: _helpers.Long.fromValue(exactTokenB.amount).toString()
        },
        slippage: slippage,
        deadline: deadline.toString()
      };
    },
    fromAmino: function fromAmino(_ref8) {
      var requester = _ref8.requester,
        token_a = _ref8.token_a,
        exact_token_b = _ref8.exact_token_b,
        slippage = _ref8.slippage,
        deadline = _ref8.deadline;
      return {
        requester: requester,
        tokenA: {
          denom: token_a.denom,
          amount: token_a.amount
        },
        exactTokenB: {
          denom: exact_token_b.denom,
          amount: exact_token_b.amount
        },
        slippage: slippage,
        deadline: _helpers.Long.fromString(deadline)
      };
    }
  }
};
exports.AminoConverter = AminoConverter;