import { CDP, Deposit } from "./cdp";
import { Coin } from "../../../cosmos/base/v1beta1/coin";
import { Timestamp } from "../../../google/protobuf/timestamp";
import * as _m0 from "protobufjs/minimal";
import { Long, isSet, fromJsonTimestamp, fromTimestamp } from "../../../helpers";
/** GenesisState defines the cdp module's genesis state. */

function createBaseGenesisState() {
  return {
    params: undefined,
    cdps: [],
    deposits: [],
    startingCdpId: Long.UZERO,
    debtDenom: "",
    govDenom: "",
    previousAccumulationTimes: [],
    totalPrincipals: []
  };
}
export const GenesisState = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.params !== undefined) {
      Params.encode(message.params, writer.uint32(10).fork()).ldelim();
    }
    for (const v of message.cdps) {
      CDP.encode(v, writer.uint32(18).fork()).ldelim();
    }
    for (const v of message.deposits) {
      Deposit.encode(v, writer.uint32(26).fork()).ldelim();
    }
    if (!message.startingCdpId.isZero()) {
      writer.uint32(32).uint64(message.startingCdpId);
    }
    if (message.debtDenom !== "") {
      writer.uint32(42).string(message.debtDenom);
    }
    if (message.govDenom !== "") {
      writer.uint32(50).string(message.govDenom);
    }
    for (const v of message.previousAccumulationTimes) {
      GenesisAccumulationTime.encode(v, writer.uint32(58).fork()).ldelim();
    }
    for (const v of message.totalPrincipals) {
      GenesisTotalPrincipal.encode(v, writer.uint32(66).fork()).ldelim();
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseGenesisState();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.params = Params.decode(reader, reader.uint32());
          break;
        case 2:
          message.cdps.push(CDP.decode(reader, reader.uint32()));
          break;
        case 3:
          message.deposits.push(Deposit.decode(reader, reader.uint32()));
          break;
        case 4:
          message.startingCdpId = reader.uint64();
          break;
        case 5:
          message.debtDenom = reader.string();
          break;
        case 6:
          message.govDenom = reader.string();
          break;
        case 7:
          message.previousAccumulationTimes.push(GenesisAccumulationTime.decode(reader, reader.uint32()));
          break;
        case 8:
          message.totalPrincipals.push(GenesisTotalPrincipal.decode(reader, reader.uint32()));
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      params: isSet(object.params) ? Params.fromJSON(object.params) : undefined,
      cdps: Array.isArray(object === null || object === void 0 ? void 0 : object.cdps) ? object.cdps.map(e => CDP.fromJSON(e)) : [],
      deposits: Array.isArray(object === null || object === void 0 ? void 0 : object.deposits) ? object.deposits.map(e => Deposit.fromJSON(e)) : [],
      startingCdpId: isSet(object.startingCdpId) ? Long.fromValue(object.startingCdpId) : Long.UZERO,
      debtDenom: isSet(object.debtDenom) ? String(object.debtDenom) : "",
      govDenom: isSet(object.govDenom) ? String(object.govDenom) : "",
      previousAccumulationTimes: Array.isArray(object === null || object === void 0 ? void 0 : object.previousAccumulationTimes) ? object.previousAccumulationTimes.map(e => GenesisAccumulationTime.fromJSON(e)) : [],
      totalPrincipals: Array.isArray(object === null || object === void 0 ? void 0 : object.totalPrincipals) ? object.totalPrincipals.map(e => GenesisTotalPrincipal.fromJSON(e)) : []
    };
  },
  toJSON(message) {
    const obj = {};
    message.params !== undefined && (obj.params = message.params ? Params.toJSON(message.params) : undefined);
    if (message.cdps) {
      obj.cdps = message.cdps.map(e => e ? CDP.toJSON(e) : undefined);
    } else {
      obj.cdps = [];
    }
    if (message.deposits) {
      obj.deposits = message.deposits.map(e => e ? Deposit.toJSON(e) : undefined);
    } else {
      obj.deposits = [];
    }
    message.startingCdpId !== undefined && (obj.startingCdpId = (message.startingCdpId || Long.UZERO).toString());
    message.debtDenom !== undefined && (obj.debtDenom = message.debtDenom);
    message.govDenom !== undefined && (obj.govDenom = message.govDenom);
    if (message.previousAccumulationTimes) {
      obj.previousAccumulationTimes = message.previousAccumulationTimes.map(e => e ? GenesisAccumulationTime.toJSON(e) : undefined);
    } else {
      obj.previousAccumulationTimes = [];
    }
    if (message.totalPrincipals) {
      obj.totalPrincipals = message.totalPrincipals.map(e => e ? GenesisTotalPrincipal.toJSON(e) : undefined);
    } else {
      obj.totalPrincipals = [];
    }
    return obj;
  },
  fromPartial(object) {
    var _object$cdps, _object$deposits, _object$debtDenom, _object$govDenom, _object$previousAccum, _object$totalPrincipa;
    const message = createBaseGenesisState();
    message.params = object.params !== undefined && object.params !== null ? Params.fromPartial(object.params) : undefined;
    message.cdps = ((_object$cdps = object.cdps) === null || _object$cdps === void 0 ? void 0 : _object$cdps.map(e => CDP.fromPartial(e))) || [];
    message.deposits = ((_object$deposits = object.deposits) === null || _object$deposits === void 0 ? void 0 : _object$deposits.map(e => Deposit.fromPartial(e))) || [];
    message.startingCdpId = object.startingCdpId !== undefined && object.startingCdpId !== null ? Long.fromValue(object.startingCdpId) : Long.UZERO;
    message.debtDenom = (_object$debtDenom = object.debtDenom) !== null && _object$debtDenom !== void 0 ? _object$debtDenom : "";
    message.govDenom = (_object$govDenom = object.govDenom) !== null && _object$govDenom !== void 0 ? _object$govDenom : "";
    message.previousAccumulationTimes = ((_object$previousAccum = object.previousAccumulationTimes) === null || _object$previousAccum === void 0 ? void 0 : _object$previousAccum.map(e => GenesisAccumulationTime.fromPartial(e))) || [];
    message.totalPrincipals = ((_object$totalPrincipa = object.totalPrincipals) === null || _object$totalPrincipa === void 0 ? void 0 : _object$totalPrincipa.map(e => GenesisTotalPrincipal.fromPartial(e))) || [];
    return message;
  }
};
function createBaseParams() {
  return {
    collateralParams: [],
    debtParam: undefined,
    globalDebtLimit: undefined,
    surplusAuctionThreshold: "",
    surplusAuctionLot: "",
    debtAuctionThreshold: "",
    debtAuctionLot: "",
    circuitBreaker: false
  };
}
export const Params = {
  encode(message, writer = _m0.Writer.create()) {
    for (const v of message.collateralParams) {
      CollateralParam.encode(v, writer.uint32(10).fork()).ldelim();
    }
    if (message.debtParam !== undefined) {
      DebtParam.encode(message.debtParam, writer.uint32(18).fork()).ldelim();
    }
    if (message.globalDebtLimit !== undefined) {
      Coin.encode(message.globalDebtLimit, writer.uint32(26).fork()).ldelim();
    }
    if (message.surplusAuctionThreshold !== "") {
      writer.uint32(34).string(message.surplusAuctionThreshold);
    }
    if (message.surplusAuctionLot !== "") {
      writer.uint32(42).string(message.surplusAuctionLot);
    }
    if (message.debtAuctionThreshold !== "") {
      writer.uint32(50).string(message.debtAuctionThreshold);
    }
    if (message.debtAuctionLot !== "") {
      writer.uint32(58).string(message.debtAuctionLot);
    }
    if (message.circuitBreaker === true) {
      writer.uint32(64).bool(message.circuitBreaker);
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseParams();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.collateralParams.push(CollateralParam.decode(reader, reader.uint32()));
          break;
        case 2:
          message.debtParam = DebtParam.decode(reader, reader.uint32());
          break;
        case 3:
          message.globalDebtLimit = Coin.decode(reader, reader.uint32());
          break;
        case 4:
          message.surplusAuctionThreshold = reader.string();
          break;
        case 5:
          message.surplusAuctionLot = reader.string();
          break;
        case 6:
          message.debtAuctionThreshold = reader.string();
          break;
        case 7:
          message.debtAuctionLot = reader.string();
          break;
        case 8:
          message.circuitBreaker = reader.bool();
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      collateralParams: Array.isArray(object === null || object === void 0 ? void 0 : object.collateralParams) ? object.collateralParams.map(e => CollateralParam.fromJSON(e)) : [],
      debtParam: isSet(object.debtParam) ? DebtParam.fromJSON(object.debtParam) : undefined,
      globalDebtLimit: isSet(object.globalDebtLimit) ? Coin.fromJSON(object.globalDebtLimit) : undefined,
      surplusAuctionThreshold: isSet(object.surplusAuctionThreshold) ? String(object.surplusAuctionThreshold) : "",
      surplusAuctionLot: isSet(object.surplusAuctionLot) ? String(object.surplusAuctionLot) : "",
      debtAuctionThreshold: isSet(object.debtAuctionThreshold) ? String(object.debtAuctionThreshold) : "",
      debtAuctionLot: isSet(object.debtAuctionLot) ? String(object.debtAuctionLot) : "",
      circuitBreaker: isSet(object.circuitBreaker) ? Boolean(object.circuitBreaker) : false
    };
  },
  toJSON(message) {
    const obj = {};
    if (message.collateralParams) {
      obj.collateralParams = message.collateralParams.map(e => e ? CollateralParam.toJSON(e) : undefined);
    } else {
      obj.collateralParams = [];
    }
    message.debtParam !== undefined && (obj.debtParam = message.debtParam ? DebtParam.toJSON(message.debtParam) : undefined);
    message.globalDebtLimit !== undefined && (obj.globalDebtLimit = message.globalDebtLimit ? Coin.toJSON(message.globalDebtLimit) : undefined);
    message.surplusAuctionThreshold !== undefined && (obj.surplusAuctionThreshold = message.surplusAuctionThreshold);
    message.surplusAuctionLot !== undefined && (obj.surplusAuctionLot = message.surplusAuctionLot);
    message.debtAuctionThreshold !== undefined && (obj.debtAuctionThreshold = message.debtAuctionThreshold);
    message.debtAuctionLot !== undefined && (obj.debtAuctionLot = message.debtAuctionLot);
    message.circuitBreaker !== undefined && (obj.circuitBreaker = message.circuitBreaker);
    return obj;
  },
  fromPartial(object) {
    var _object$collateralPar, _object$surplusAuctio, _object$surplusAuctio2, _object$debtAuctionTh, _object$debtAuctionLo, _object$circuitBreake;
    const message = createBaseParams();
    message.collateralParams = ((_object$collateralPar = object.collateralParams) === null || _object$collateralPar === void 0 ? void 0 : _object$collateralPar.map(e => CollateralParam.fromPartial(e))) || [];
    message.debtParam = object.debtParam !== undefined && object.debtParam !== null ? DebtParam.fromPartial(object.debtParam) : undefined;
    message.globalDebtLimit = object.globalDebtLimit !== undefined && object.globalDebtLimit !== null ? Coin.fromPartial(object.globalDebtLimit) : undefined;
    message.surplusAuctionThreshold = (_object$surplusAuctio = object.surplusAuctionThreshold) !== null && _object$surplusAuctio !== void 0 ? _object$surplusAuctio : "";
    message.surplusAuctionLot = (_object$surplusAuctio2 = object.surplusAuctionLot) !== null && _object$surplusAuctio2 !== void 0 ? _object$surplusAuctio2 : "";
    message.debtAuctionThreshold = (_object$debtAuctionTh = object.debtAuctionThreshold) !== null && _object$debtAuctionTh !== void 0 ? _object$debtAuctionTh : "";
    message.debtAuctionLot = (_object$debtAuctionLo = object.debtAuctionLot) !== null && _object$debtAuctionLo !== void 0 ? _object$debtAuctionLo : "";
    message.circuitBreaker = (_object$circuitBreake = object.circuitBreaker) !== null && _object$circuitBreake !== void 0 ? _object$circuitBreake : false;
    return message;
  }
};
function createBaseDebtParam() {
  return {
    denom: "",
    referenceAsset: "",
    conversionFactor: "",
    debtFloor: ""
  };
}
export const DebtParam = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.denom !== "") {
      writer.uint32(10).string(message.denom);
    }
    if (message.referenceAsset !== "") {
      writer.uint32(18).string(message.referenceAsset);
    }
    if (message.conversionFactor !== "") {
      writer.uint32(26).string(message.conversionFactor);
    }
    if (message.debtFloor !== "") {
      writer.uint32(34).string(message.debtFloor);
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseDebtParam();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.denom = reader.string();
          break;
        case 2:
          message.referenceAsset = reader.string();
          break;
        case 3:
          message.conversionFactor = reader.string();
          break;
        case 4:
          message.debtFloor = reader.string();
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      denom: isSet(object.denom) ? String(object.denom) : "",
      referenceAsset: isSet(object.referenceAsset) ? String(object.referenceAsset) : "",
      conversionFactor: isSet(object.conversionFactor) ? String(object.conversionFactor) : "",
      debtFloor: isSet(object.debtFloor) ? String(object.debtFloor) : ""
    };
  },
  toJSON(message) {
    const obj = {};
    message.denom !== undefined && (obj.denom = message.denom);
    message.referenceAsset !== undefined && (obj.referenceAsset = message.referenceAsset);
    message.conversionFactor !== undefined && (obj.conversionFactor = message.conversionFactor);
    message.debtFloor !== undefined && (obj.debtFloor = message.debtFloor);
    return obj;
  },
  fromPartial(object) {
    var _object$denom, _object$referenceAsse, _object$conversionFac, _object$debtFloor;
    const message = createBaseDebtParam();
    message.denom = (_object$denom = object.denom) !== null && _object$denom !== void 0 ? _object$denom : "";
    message.referenceAsset = (_object$referenceAsse = object.referenceAsset) !== null && _object$referenceAsse !== void 0 ? _object$referenceAsse : "";
    message.conversionFactor = (_object$conversionFac = object.conversionFactor) !== null && _object$conversionFac !== void 0 ? _object$conversionFac : "";
    message.debtFloor = (_object$debtFloor = object.debtFloor) !== null && _object$debtFloor !== void 0 ? _object$debtFloor : "";
    return message;
  }
};
function createBaseCollateralParam() {
  return {
    denom: "",
    type: "",
    liquidationRatio: "",
    debtLimit: undefined,
    stabilityFee: "",
    auctionSize: "",
    liquidationPenalty: "",
    spotMarketId: "",
    liquidationMarketId: "",
    keeperRewardPercentage: "",
    checkCollateralizationIndexCount: "",
    conversionFactor: ""
  };
}
export const CollateralParam = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.denom !== "") {
      writer.uint32(10).string(message.denom);
    }
    if (message.type !== "") {
      writer.uint32(18).string(message.type);
    }
    if (message.liquidationRatio !== "") {
      writer.uint32(26).string(message.liquidationRatio);
    }
    if (message.debtLimit !== undefined) {
      Coin.encode(message.debtLimit, writer.uint32(34).fork()).ldelim();
    }
    if (message.stabilityFee !== "") {
      writer.uint32(42).string(message.stabilityFee);
    }
    if (message.auctionSize !== "") {
      writer.uint32(50).string(message.auctionSize);
    }
    if (message.liquidationPenalty !== "") {
      writer.uint32(58).string(message.liquidationPenalty);
    }
    if (message.spotMarketId !== "") {
      writer.uint32(66).string(message.spotMarketId);
    }
    if (message.liquidationMarketId !== "") {
      writer.uint32(74).string(message.liquidationMarketId);
    }
    if (message.keeperRewardPercentage !== "") {
      writer.uint32(82).string(message.keeperRewardPercentage);
    }
    if (message.checkCollateralizationIndexCount !== "") {
      writer.uint32(90).string(message.checkCollateralizationIndexCount);
    }
    if (message.conversionFactor !== "") {
      writer.uint32(98).string(message.conversionFactor);
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseCollateralParam();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.denom = reader.string();
          break;
        case 2:
          message.type = reader.string();
          break;
        case 3:
          message.liquidationRatio = reader.string();
          break;
        case 4:
          message.debtLimit = Coin.decode(reader, reader.uint32());
          break;
        case 5:
          message.stabilityFee = reader.string();
          break;
        case 6:
          message.auctionSize = reader.string();
          break;
        case 7:
          message.liquidationPenalty = reader.string();
          break;
        case 8:
          message.spotMarketId = reader.string();
          break;
        case 9:
          message.liquidationMarketId = reader.string();
          break;
        case 10:
          message.keeperRewardPercentage = reader.string();
          break;
        case 11:
          message.checkCollateralizationIndexCount = reader.string();
          break;
        case 12:
          message.conversionFactor = reader.string();
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      denom: isSet(object.denom) ? String(object.denom) : "",
      type: isSet(object.type) ? String(object.type) : "",
      liquidationRatio: isSet(object.liquidationRatio) ? String(object.liquidationRatio) : "",
      debtLimit: isSet(object.debtLimit) ? Coin.fromJSON(object.debtLimit) : undefined,
      stabilityFee: isSet(object.stabilityFee) ? String(object.stabilityFee) : "",
      auctionSize: isSet(object.auctionSize) ? String(object.auctionSize) : "",
      liquidationPenalty: isSet(object.liquidationPenalty) ? String(object.liquidationPenalty) : "",
      spotMarketId: isSet(object.spotMarketId) ? String(object.spotMarketId) : "",
      liquidationMarketId: isSet(object.liquidationMarketId) ? String(object.liquidationMarketId) : "",
      keeperRewardPercentage: isSet(object.keeperRewardPercentage) ? String(object.keeperRewardPercentage) : "",
      checkCollateralizationIndexCount: isSet(object.checkCollateralizationIndexCount) ? String(object.checkCollateralizationIndexCount) : "",
      conversionFactor: isSet(object.conversionFactor) ? String(object.conversionFactor) : ""
    };
  },
  toJSON(message) {
    const obj = {};
    message.denom !== undefined && (obj.denom = message.denom);
    message.type !== undefined && (obj.type = message.type);
    message.liquidationRatio !== undefined && (obj.liquidationRatio = message.liquidationRatio);
    message.debtLimit !== undefined && (obj.debtLimit = message.debtLimit ? Coin.toJSON(message.debtLimit) : undefined);
    message.stabilityFee !== undefined && (obj.stabilityFee = message.stabilityFee);
    message.auctionSize !== undefined && (obj.auctionSize = message.auctionSize);
    message.liquidationPenalty !== undefined && (obj.liquidationPenalty = message.liquidationPenalty);
    message.spotMarketId !== undefined && (obj.spotMarketId = message.spotMarketId);
    message.liquidationMarketId !== undefined && (obj.liquidationMarketId = message.liquidationMarketId);
    message.keeperRewardPercentage !== undefined && (obj.keeperRewardPercentage = message.keeperRewardPercentage);
    message.checkCollateralizationIndexCount !== undefined && (obj.checkCollateralizationIndexCount = message.checkCollateralizationIndexCount);
    message.conversionFactor !== undefined && (obj.conversionFactor = message.conversionFactor);
    return obj;
  },
  fromPartial(object) {
    var _object$denom2, _object$type, _object$liquidationRa, _object$stabilityFee, _object$auctionSize, _object$liquidationPe, _object$spotMarketId, _object$liquidationMa, _object$keeperRewardP, _object$checkCollater, _object$conversionFac2;
    const message = createBaseCollateralParam();
    message.denom = (_object$denom2 = object.denom) !== null && _object$denom2 !== void 0 ? _object$denom2 : "";
    message.type = (_object$type = object.type) !== null && _object$type !== void 0 ? _object$type : "";
    message.liquidationRatio = (_object$liquidationRa = object.liquidationRatio) !== null && _object$liquidationRa !== void 0 ? _object$liquidationRa : "";
    message.debtLimit = object.debtLimit !== undefined && object.debtLimit !== null ? Coin.fromPartial(object.debtLimit) : undefined;
    message.stabilityFee = (_object$stabilityFee = object.stabilityFee) !== null && _object$stabilityFee !== void 0 ? _object$stabilityFee : "";
    message.auctionSize = (_object$auctionSize = object.auctionSize) !== null && _object$auctionSize !== void 0 ? _object$auctionSize : "";
    message.liquidationPenalty = (_object$liquidationPe = object.liquidationPenalty) !== null && _object$liquidationPe !== void 0 ? _object$liquidationPe : "";
    message.spotMarketId = (_object$spotMarketId = object.spotMarketId) !== null && _object$spotMarketId !== void 0 ? _object$spotMarketId : "";
    message.liquidationMarketId = (_object$liquidationMa = object.liquidationMarketId) !== null && _object$liquidationMa !== void 0 ? _object$liquidationMa : "";
    message.keeperRewardPercentage = (_object$keeperRewardP = object.keeperRewardPercentage) !== null && _object$keeperRewardP !== void 0 ? _object$keeperRewardP : "";
    message.checkCollateralizationIndexCount = (_object$checkCollater = object.checkCollateralizationIndexCount) !== null && _object$checkCollater !== void 0 ? _object$checkCollater : "";
    message.conversionFactor = (_object$conversionFac2 = object.conversionFactor) !== null && _object$conversionFac2 !== void 0 ? _object$conversionFac2 : "";
    return message;
  }
};
function createBaseGenesisAccumulationTime() {
  return {
    collateralType: "",
    previousAccumulationTime: undefined,
    interestFactor: ""
  };
}
export const GenesisAccumulationTime = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.collateralType !== "") {
      writer.uint32(10).string(message.collateralType);
    }
    if (message.previousAccumulationTime !== undefined) {
      Timestamp.encode(message.previousAccumulationTime, writer.uint32(18).fork()).ldelim();
    }
    if (message.interestFactor !== "") {
      writer.uint32(26).string(message.interestFactor);
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseGenesisAccumulationTime();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.collateralType = reader.string();
          break;
        case 2:
          message.previousAccumulationTime = Timestamp.decode(reader, reader.uint32());
          break;
        case 3:
          message.interestFactor = reader.string();
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      collateralType: isSet(object.collateralType) ? String(object.collateralType) : "",
      previousAccumulationTime: isSet(object.previousAccumulationTime) ? fromJsonTimestamp(object.previousAccumulationTime) : undefined,
      interestFactor: isSet(object.interestFactor) ? String(object.interestFactor) : ""
    };
  },
  toJSON(message) {
    const obj = {};
    message.collateralType !== undefined && (obj.collateralType = message.collateralType);
    message.previousAccumulationTime !== undefined && (obj.previousAccumulationTime = fromTimestamp(message.previousAccumulationTime).toISOString());
    message.interestFactor !== undefined && (obj.interestFactor = message.interestFactor);
    return obj;
  },
  fromPartial(object) {
    var _object$collateralTyp, _object$interestFacto;
    const message = createBaseGenesisAccumulationTime();
    message.collateralType = (_object$collateralTyp = object.collateralType) !== null && _object$collateralTyp !== void 0 ? _object$collateralTyp : "";
    message.previousAccumulationTime = object.previousAccumulationTime !== undefined && object.previousAccumulationTime !== null ? Timestamp.fromPartial(object.previousAccumulationTime) : undefined;
    message.interestFactor = (_object$interestFacto = object.interestFactor) !== null && _object$interestFacto !== void 0 ? _object$interestFacto : "";
    return message;
  }
};
function createBaseGenesisTotalPrincipal() {
  return {
    collateralType: "",
    totalPrincipal: ""
  };
}
export const GenesisTotalPrincipal = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.collateralType !== "") {
      writer.uint32(10).string(message.collateralType);
    }
    if (message.totalPrincipal !== "") {
      writer.uint32(18).string(message.totalPrincipal);
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseGenesisTotalPrincipal();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.collateralType = reader.string();
          break;
        case 2:
          message.totalPrincipal = reader.string();
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      collateralType: isSet(object.collateralType) ? String(object.collateralType) : "",
      totalPrincipal: isSet(object.totalPrincipal) ? String(object.totalPrincipal) : ""
    };
  },
  toJSON(message) {
    const obj = {};
    message.collateralType !== undefined && (obj.collateralType = message.collateralType);
    message.totalPrincipal !== undefined && (obj.totalPrincipal = message.totalPrincipal);
    return obj;
  },
  fromPartial(object) {
    var _object$collateralTyp2, _object$totalPrincipa2;
    const message = createBaseGenesisTotalPrincipal();
    message.collateralType = (_object$collateralTyp2 = object.collateralType) !== null && _object$collateralTyp2 !== void 0 ? _object$collateralTyp2 : "";
    message.totalPrincipal = (_object$totalPrincipa2 = object.totalPrincipal) !== null && _object$totalPrincipa2 !== void 0 ? _object$totalPrincipa2 : "";
    return message;
  }
};