import { strategyTypeFromJSON, strategyTypeToJSON } from "./strategy";
import * as _m0 from "protobufjs/minimal";
import { isSet, bytesFromBase64, base64FromBytes } from "../../../helpers";
/**
 * AllowedVault is a vault that is allowed to be created. These can be
 * modified via parameter governance.
 */

function createBaseAllowedVault() {
  return {
    denom: "",
    strategies: [],
    isPrivateVault: false,
    allowedDepositors: []
  };
}
export const AllowedVault = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.denom !== "") {
      writer.uint32(10).string(message.denom);
    }
    writer.uint32(18).fork();
    for (const v of message.strategies) {
      writer.int32(v);
    }
    writer.ldelim();
    if (message.isPrivateVault === true) {
      writer.uint32(24).bool(message.isPrivateVault);
    }
    for (const v of message.allowedDepositors) {
      writer.uint32(34).bytes(v);
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseAllowedVault();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.denom = reader.string();
          break;
        case 2:
          if ((tag & 7) === 2) {
            const end2 = reader.uint32() + reader.pos;
            while (reader.pos < end2) {
              message.strategies.push(reader.int32());
            }
          } else {
            message.strategies.push(reader.int32());
          }
          break;
        case 3:
          message.isPrivateVault = reader.bool();
          break;
        case 4:
          message.allowedDepositors.push(reader.bytes());
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      denom: isSet(object.denom) ? String(object.denom) : "",
      strategies: Array.isArray(object === null || object === void 0 ? void 0 : object.strategies) ? object.strategies.map(e => strategyTypeFromJSON(e)) : [],
      isPrivateVault: isSet(object.isPrivateVault) ? Boolean(object.isPrivateVault) : false,
      allowedDepositors: Array.isArray(object === null || object === void 0 ? void 0 : object.allowedDepositors) ? object.allowedDepositors.map(e => bytesFromBase64(e)) : []
    };
  },
  toJSON(message) {
    const obj = {};
    message.denom !== undefined && (obj.denom = message.denom);
    if (message.strategies) {
      obj.strategies = message.strategies.map(e => strategyTypeToJSON(e));
    } else {
      obj.strategies = [];
    }
    message.isPrivateVault !== undefined && (obj.isPrivateVault = message.isPrivateVault);
    if (message.allowedDepositors) {
      obj.allowedDepositors = message.allowedDepositors.map(e => base64FromBytes(e !== undefined ? e : new Uint8Array()));
    } else {
      obj.allowedDepositors = [];
    }
    return obj;
  },
  fromPartial(object) {
    var _object$denom, _object$strategies, _object$isPrivateVaul, _object$allowedDeposi;
    const message = createBaseAllowedVault();
    message.denom = (_object$denom = object.denom) !== null && _object$denom !== void 0 ? _object$denom : "";
    message.strategies = ((_object$strategies = object.strategies) === null || _object$strategies === void 0 ? void 0 : _object$strategies.map(e => e)) || [];
    message.isPrivateVault = (_object$isPrivateVaul = object.isPrivateVault) !== null && _object$isPrivateVaul !== void 0 ? _object$isPrivateVaul : false;
    message.allowedDepositors = ((_object$allowedDeposi = object.allowedDepositors) === null || _object$allowedDeposi === void 0 ? void 0 : _object$allowedDeposi.map(e => e)) || [];
    return message;
  }
};
function createBaseVaultRecord() {
  return {
    totalShares: undefined
  };
}
export const VaultRecord = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.totalShares !== undefined) {
      VaultShare.encode(message.totalShares, writer.uint32(10).fork()).ldelim();
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseVaultRecord();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.totalShares = VaultShare.decode(reader, reader.uint32());
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      totalShares: isSet(object.totalShares) ? VaultShare.fromJSON(object.totalShares) : undefined
    };
  },
  toJSON(message) {
    const obj = {};
    message.totalShares !== undefined && (obj.totalShares = message.totalShares ? VaultShare.toJSON(message.totalShares) : undefined);
    return obj;
  },
  fromPartial(object) {
    const message = createBaseVaultRecord();
    message.totalShares = object.totalShares !== undefined && object.totalShares !== null ? VaultShare.fromPartial(object.totalShares) : undefined;
    return message;
  }
};
function createBaseVaultShareRecord() {
  return {
    depositor: new Uint8Array(),
    shares: []
  };
}
export const VaultShareRecord = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.depositor.length !== 0) {
      writer.uint32(10).bytes(message.depositor);
    }
    for (const v of message.shares) {
      VaultShare.encode(v, writer.uint32(18).fork()).ldelim();
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseVaultShareRecord();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.depositor = reader.bytes();
          break;
        case 2:
          message.shares.push(VaultShare.decode(reader, reader.uint32()));
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      depositor: isSet(object.depositor) ? bytesFromBase64(object.depositor) : new Uint8Array(),
      shares: Array.isArray(object === null || object === void 0 ? void 0 : object.shares) ? object.shares.map(e => VaultShare.fromJSON(e)) : []
    };
  },
  toJSON(message) {
    const obj = {};
    message.depositor !== undefined && (obj.depositor = base64FromBytes(message.depositor !== undefined ? message.depositor : new Uint8Array()));
    if (message.shares) {
      obj.shares = message.shares.map(e => e ? VaultShare.toJSON(e) : undefined);
    } else {
      obj.shares = [];
    }
    return obj;
  },
  fromPartial(object) {
    var _object$depositor, _object$shares;
    const message = createBaseVaultShareRecord();
    message.depositor = (_object$depositor = object.depositor) !== null && _object$depositor !== void 0 ? _object$depositor : new Uint8Array();
    message.shares = ((_object$shares = object.shares) === null || _object$shares === void 0 ? void 0 : _object$shares.map(e => VaultShare.fromPartial(e))) || [];
    return message;
  }
};
function createBaseVaultShare() {
  return {
    denom: "",
    amount: ""
  };
}
export const VaultShare = {
  encode(message, writer = _m0.Writer.create()) {
    if (message.denom !== "") {
      writer.uint32(10).string(message.denom);
    }
    if (message.amount !== "") {
      writer.uint32(18).string(message.amount);
    }
    return writer;
  },
  decode(input, length) {
    const reader = input instanceof _m0.Reader ? input : new _m0.Reader(input);
    let end = length === undefined ? reader.len : reader.pos + length;
    const message = createBaseVaultShare();
    while (reader.pos < end) {
      const tag = reader.uint32();
      switch (tag >>> 3) {
        case 1:
          message.denom = reader.string();
          break;
        case 2:
          message.amount = reader.string();
          break;
        default:
          reader.skipType(tag & 7);
          break;
      }
    }
    return message;
  },
  fromJSON(object) {
    return {
      denom: isSet(object.denom) ? String(object.denom) : "",
      amount: isSet(object.amount) ? String(object.amount) : ""
    };
  },
  toJSON(message) {
    const obj = {};
    message.denom !== undefined && (obj.denom = message.denom);
    message.amount !== undefined && (obj.amount = message.amount);
    return obj;
  },
  fromPartial(object) {
    var _object$denom2, _object$amount;
    const message = createBaseVaultShare();
    message.denom = (_object$denom2 = object.denom) !== null && _object$denom2 !== void 0 ? _object$denom2 : "";
    message.amount = (_object$amount = object.amount) !== null && _object$amount !== void 0 ? _object$amount : "";
    return message;
  }
};