import { Any, AnySDKType } from "../../../google/protobuf/any";
import { AccessTuple, AccessTupleSDKType, Log, LogSDKType } from "./evm";
import * as _m0 from "protobufjs/minimal";
import { Long } from "../../../helpers";
/** MsgEthereumTx encapsulates an Ethereum transaction as an SDK message. */
export interface MsgEthereumTx {
    /** inner transaction data */
    data?: Any;
    /** encoded storage size of the transaction */
    size: number;
    /** transaction hash in hex format */
    hash: string;
    /**
     * ethereum signer address in hex format. This address value is checked
     * against the address derived from the signature (V, R, S) using the
     * secp256k1 elliptic curve
     */
    from: string;
}
/** MsgEthereumTx encapsulates an Ethereum transaction as an SDK message. */
export interface MsgEthereumTxSDKType {
    /** inner transaction data */
    data?: AnySDKType;
    /** encoded storage size of the transaction */
    size: number;
    /** transaction hash in hex format */
    hash: string;
    /**
     * ethereum signer address in hex format. This address value is checked
     * against the address derived from the signature (V, R, S) using the
     * secp256k1 elliptic curve
     */
    from: string;
}
/** LegacyTx is the transaction data of regular Ethereum transactions. */
export interface LegacyTx {
    /** nonce corresponds to the account nonce (transaction sequence). */
    nonce: Long;
    /** gas price defines the value for each gas unit */
    gasPrice: string;
    /** gas defines the gas limit defined for the transaction. */
    gas: Long;
    /** hex formatted address of the recipient */
    to: string;
    /** value defines the unsigned integer value of the transaction amount. */
    value: string;
    /** input defines the data payload bytes of the transaction. */
    data: Uint8Array;
    /** v defines the signature value */
    v: Uint8Array;
    /** r defines the signature value */
    r: Uint8Array;
    /** s define the signature value */
    s: Uint8Array;
}
/** LegacyTx is the transaction data of regular Ethereum transactions. */
export interface LegacyTxSDKType {
    /** nonce corresponds to the account nonce (transaction sequence). */
    nonce: Long;
    /** gas price defines the value for each gas unit */
    gas_price: string;
    /** gas defines the gas limit defined for the transaction. */
    gas: Long;
    /** hex formatted address of the recipient */
    to: string;
    /** value defines the unsigned integer value of the transaction amount. */
    value: string;
    /** input defines the data payload bytes of the transaction. */
    data: Uint8Array;
    /** v defines the signature value */
    v: Uint8Array;
    /** r defines the signature value */
    r: Uint8Array;
    /** s define the signature value */
    s: Uint8Array;
}
/** AccessListTx is the data of EIP-2930 access list transactions. */
export interface AccessListTx {
    /** destination EVM chain ID */
    chainId: string;
    /** nonce corresponds to the account nonce (transaction sequence). */
    nonce: Long;
    /** gas price defines the value for each gas unit */
    gasPrice: string;
    /** gas defines the gas limit defined for the transaction. */
    gas: Long;
    /** hex formatted address of the recipient */
    to: string;
    /** value defines the unsigned integer value of the transaction amount. */
    value: string;
    /** input defines the data payload bytes of the transaction. */
    data: Uint8Array;
    accesses: AccessTuple[];
    /** v defines the signature value */
    v: Uint8Array;
    /** r defines the signature value */
    r: Uint8Array;
    /** s define the signature value */
    s: Uint8Array;
}
/** AccessListTx is the data of EIP-2930 access list transactions. */
export interface AccessListTxSDKType {
    /** destination EVM chain ID */
    chain_id: string;
    /** nonce corresponds to the account nonce (transaction sequence). */
    nonce: Long;
    /** gas price defines the value for each gas unit */
    gas_price: string;
    /** gas defines the gas limit defined for the transaction. */
    gas: Long;
    /** hex formatted address of the recipient */
    to: string;
    /** value defines the unsigned integer value of the transaction amount. */
    value: string;
    /** input defines the data payload bytes of the transaction. */
    data: Uint8Array;
    accesses: AccessTupleSDKType[];
    /** v defines the signature value */
    v: Uint8Array;
    /** r defines the signature value */
    r: Uint8Array;
    /** s define the signature value */
    s: Uint8Array;
}
/** DynamicFeeTx is the data of EIP-1559 dinamic fee transactions. */
export interface DynamicFeeTx {
    /** destination EVM chain ID */
    chainId: string;
    /** nonce corresponds to the account nonce (transaction sequence). */
    nonce: Long;
    /** gas tip cap defines the max value for the gas tip */
    gasTipCap: string;
    /** gas fee cap defines the max value for the gas fee */
    gasFeeCap: string;
    /** gas defines the gas limit defined for the transaction. */
    gas: Long;
    /** hex formatted address of the recipient */
    to: string;
    /** value defines the the transaction amount. */
    value: string;
    /** input defines the data payload bytes of the transaction. */
    data: Uint8Array;
    accesses: AccessTuple[];
    /** v defines the signature value */
    v: Uint8Array;
    /** r defines the signature value */
    r: Uint8Array;
    /** s define the signature value */
    s: Uint8Array;
}
/** DynamicFeeTx is the data of EIP-1559 dinamic fee transactions. */
export interface DynamicFeeTxSDKType {
    /** destination EVM chain ID */
    chain_id: string;
    /** nonce corresponds to the account nonce (transaction sequence). */
    nonce: Long;
    /** gas tip cap defines the max value for the gas tip */
    gas_tip_cap: string;
    /** gas fee cap defines the max value for the gas fee */
    gas_fee_cap: string;
    /** gas defines the gas limit defined for the transaction. */
    gas: Long;
    /** hex formatted address of the recipient */
    to: string;
    /** value defines the the transaction amount. */
    value: string;
    /** input defines the data payload bytes of the transaction. */
    data: Uint8Array;
    accesses: AccessTupleSDKType[];
    /** v defines the signature value */
    v: Uint8Array;
    /** r defines the signature value */
    r: Uint8Array;
    /** s define the signature value */
    s: Uint8Array;
}
export interface ExtensionOptionsEthereumTx {
}
export interface ExtensionOptionsEthereumTxSDKType {
}
/** MsgEthereumTxResponse defines the Msg/EthereumTx response type. */
export interface MsgEthereumTxResponse {
    /**
     * ethereum transaction hash in hex format. This hash differs from the
     * Tendermint sha256 hash of the transaction bytes. See
     * https://github.com/tendermint/tendermint/issues/6539 for reference
     */
    hash: string;
    /**
     * logs contains the transaction hash and the proto-compatible ethereum
     * logs.
     */
    logs: Log[];
    /**
     * returned data from evm function (result or data supplied with revert
     * opcode)
     */
    ret: Uint8Array;
    /** vm error is the error returned by vm execution */
    vmError: string;
    /** gas consumed by the transaction */
    gasUsed: Long;
}
/** MsgEthereumTxResponse defines the Msg/EthereumTx response type. */
export interface MsgEthereumTxResponseSDKType {
    /**
     * ethereum transaction hash in hex format. This hash differs from the
     * Tendermint sha256 hash of the transaction bytes. See
     * https://github.com/tendermint/tendermint/issues/6539 for reference
     */
    hash: string;
    /**
     * logs contains the transaction hash and the proto-compatible ethereum
     * logs.
     */
    logs: LogSDKType[];
    /**
     * returned data from evm function (result or data supplied with revert
     * opcode)
     */
    ret: Uint8Array;
    /** vm error is the error returned by vm execution */
    vm_error: string;
    /** gas consumed by the transaction */
    gas_used: Long;
}
export declare const MsgEthereumTx: {
    encode(message: MsgEthereumTx, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): MsgEthereumTx;
    fromJSON(object: any): MsgEthereumTx;
    toJSON(message: MsgEthereumTx): unknown;
    fromPartial(object: Partial<MsgEthereumTx>): MsgEthereumTx;
};
export declare const LegacyTx: {
    encode(message: LegacyTx, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): LegacyTx;
    fromJSON(object: any): LegacyTx;
    toJSON(message: LegacyTx): unknown;
    fromPartial(object: Partial<LegacyTx>): LegacyTx;
};
export declare const AccessListTx: {
    encode(message: AccessListTx, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): AccessListTx;
    fromJSON(object: any): AccessListTx;
    toJSON(message: AccessListTx): unknown;
    fromPartial(object: Partial<AccessListTx>): AccessListTx;
};
export declare const DynamicFeeTx: {
    encode(message: DynamicFeeTx, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): DynamicFeeTx;
    fromJSON(object: any): DynamicFeeTx;
    toJSON(message: DynamicFeeTx): unknown;
    fromPartial(object: Partial<DynamicFeeTx>): DynamicFeeTx;
};
export declare const ExtensionOptionsEthereumTx: {
    encode(_: ExtensionOptionsEthereumTx, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): ExtensionOptionsEthereumTx;
    fromJSON(_: any): ExtensionOptionsEthereumTx;
    toJSON(_: ExtensionOptionsEthereumTx): unknown;
    fromPartial(_: Partial<ExtensionOptionsEthereumTx>): ExtensionOptionsEthereumTx;
};
export declare const MsgEthereumTxResponse: {
    encode(message: MsgEthereumTxResponse, writer?: _m0.Writer): _m0.Writer;
    decode(input: _m0.Reader | Uint8Array, length?: number): MsgEthereumTxResponse;
    fromJSON(object: any): MsgEthereumTxResponse;
    toJSON(message: MsgEthereumTxResponse): unknown;
    fromPartial(object: Partial<MsgEthereumTxResponse>): MsgEthereumTxResponse;
};
